/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.registry.extensions.handlers;

import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.jdbc.handlers.Handler;
import org.wso2.carbon.registry.core.jdbc.handlers.RequestContext;
import org.wso2.carbon.registry.extensions.handlers.utils.ConflictResolutionReader;
import org.wso2.carbon.registry.extensions.utils.CommonUtil;

import java.io.Reader;

public class ConflictResolutionHandler extends Handler {
    @Override
    public void restore(RequestContext requestContext) throws RegistryException {
        if (!CommonUtil.isRestoringLockAvailable()) {
            return;
        }
        CommonUtil.acquireRestoringLock();
        try {
            Registry registry = requestContext.getRegistry();
            Reader reader = requestContext.getDumpingReader();
            String path = requestContext.getResourcePath().getPath();

            ConflictResolutionReader conflictResolutionReader = new ConflictResolutionReader(reader, path, registry);

            registry.restore(path,conflictResolutionReader);
            requestContext.setProcessingComplete(true);
        } finally {
            CommonUtil.releaseRestoringLock();
        }
    }
}
